#!/bin/sh
set +e
#============================================================================#
#             Recordings mix for Asterisk Monitor application                #
#============================================================================#
# (C) Denis Smirnov   <ds@seiros.ru>                             01 Feb 2008 #
# Updated Boris Gulay <boris@seiros.ru>							 19 Jun 2008 #
#============================================================================#
show_usage()
{
    echo "Use: pbx monitormix [-m] [-f format] [-p pan] [-v] left right outfile" > /dev/stderr
    echo " -p pan    -- for stereo mix, 0 <= pan <= 1" > /dev/stderr
    echo " -v        -- volume normalization" > /dev/stderr
    echo " -m        -- mono output" > /dev/stderr
    echo " -f format -- format for output file" > /dev/stderr
    echo "              speex 	-- Speex audio codec (lossy)" > /dev/stderr
    echo "              gsmwav	-- wav GSM (lossy,mono)" > /dev/stderr
    echo "              gsm		-- GSM (lossy,mono)" > /dev/stderr
    echo "              adpcm 	-- wav IMA-ADPCM (lossy)" > /dev/stderr
    echo "              mp3   	-- MPEG Layer 3 (lossy)" > /dev/stderr
    echo "              ogg   	-- OGG Vorbis (lossy)" > /dev/stderr
    echo "              wav   	-- wav (losless,default)" > /dev/stderr
    exit -1
}

guess_filetype()
{
    ext=`echo "$1" | sed 's/^.*\.\(.*\)/\1/'`
    case "$ext" in
		spx)   format=speex	 ;;
		gsm)   format=gsm    ;;
		wav)   format=wav    ;;
		mp3)   format=mp3    ;;
	    ogg)   format=ogg	 ;;
    esac

	echo "Format for output file choosed by extension: $format"
}

sox_get_vol()
{
    sox -V0 -t wav "$1" -t null /dev/null stat -v 2>&1
}

die()
{
    echo $@ > /dev/stderr
    exit -1
}

TEMP=`getopt -n monitormix -o p:vf:m -- $@` || show_usage
pan=
mono=
vol=0
format=
eval set -- $TEMP
while :; do
    case "$1" in
	-p) shift; pan=$1
	    ;;
	# volume autoadjustment
	-v) vol=1;
	    ;;
	-m) mono=1;
	    ;;
	-f) shift; format=$1
	    case "$format" in
			speex)  ;;
			gsm)    ;;
			gsmwav)	;;
			adpcm)  ;;
			wav)	;;
			mp3)	;;
			ogg)	;;
			*) die "Unknown format: '$format'"	;;
	    esac
	    ;;
	--) shift; break
	    ;;
	*) echo ">$1<"
	    ;;
    esac
    shift
done

infile1=$1
infile2=$2
outfile=`echo "$3" | sed 's/\(.*\)\..*/\1/'`

if [ -z "$1" ]; then
    show_usage
fi

if [ -z "$2" ]; then
    show_usage
fi

if [ -z "$3" ]; then
    show_usage
fi

if [ ! -f "$infile1" ]; then
    die "Input file 1 not exists: $infile1"
fi

if [ ! -f "$infile2" ]; then
    die "Input file 2 not exists: $infile2"
fi

# Try to detect format if not set
if [ -z "$format" ]; then
    guess_filetype "$outfile"
fi

# GSM will be anyway mono
if [ $format ==  "gsm" -o $format == "gsmwav" ]; then
	[ -z "$mono" ] && echo "format $format implies -m option"
	mono=1
fi	

# Test options
[ ! -z "$mono" ] && [ ! -z "$pan" ] && die "-p and -m options are incompatible"

# Set default
[ -z "$pan"    ] && pan=1
[ -z "$format" ] && format=wav && echo "Output file format will be wav"


# set volumes for normalize
if [ $vol == 1 ]; then
    vol1=`sox_get_vol "$infile1"`
    vol2=`sox_get_vol "$infile2"`
fi

if [ -z "$mono" ]; then
	mixType=M
else
	mixType=m
fi

if [ $vol == 1]
then
    mainPart="sox -V0 -$mixType -t wav -v $vol1 $infile1 -t wav -v $vol2 $infile2"
else
    mainPart="sox -V0 -$mixType -t wav $infile1 -t wav $infile2"
fi

# mixing
case $format in
    gsm)
		$mainPart -t gsm "$outfile".gsm
		;;
    gsmwav)
		$mainPart -t wav -g "$outfile".gsm.wav
		;;
    wav)
		$mainPart -t wav "$outfile".wav
		;;
	adpcm)
		$mainPart -t wav -i "$outfile".adpcm.wav
		;;
    mp3)
		$mainPart -t mp3 "$outfile".mp3
		;;
	ogg)
		$mainPart -t ogg "$outfile".ogg
		;;
    speex)
		$mainPart -t wav - | speexenc -n --vbr --dtx - "$outfile".spx > /dev/null 2> /dev/null
		;;
	*)
		die "Unknown format!"
		;;
esac

rm -f "$infile1"
rm -f "$infile2"
